local name, addon = ...;
local L = addon.L
-- Demolition and reconstruction of the existing UI elements
do
	CharacterModelFrame:Hide() -- this is the 3d Model, plus the buttons to rotate it
	CharacterResistanceFrame:Hide() -- the five resist icons over the 3d model

	CharacterAttributesFrame:Hide() -- this contains the two dropdown panels below the 3d model
		
	-- The ammo slot is the wrong size, fix before we rescale the whole lot of em
	CharacterAmmoSlot:SetHeight(37)
	CharacterAmmoSlot:SetWidth(37)
	CharacterAmmoSlotNormalTexture:SetHeight(64)
	CharacterAmmoSlotNormalTexture:SetWidth(64)
end


local CE_PaperDollFrame = CreateFrame("Frame", nil, PaperDollFrame)
CE_PaperDollFrame:SetPoint("TOPLEFT", CharacterHeadSlot, "TOPRIGHT", 10, -4)
CE_PaperDollFrame:SetPoint("TOPRIGHT", CharacterHandsSlot, "TOPLEFT", -2, -4)
CE_PaperDollFrame:SetPoint("BOTTOM", CharacterMainHandSlot, "TOP", 0, 16)
CE_PaperDollFrame:Hide()

PaperDollFrame_OnLoad(CE_PaperDollFrame)
CE_PaperDollFrame:SetScript("OnEvent", CE_PaperDollFrame_OnEvent)


-- Build the rows
local rows = {}
local NUMROWS, GAP = 20, 0
local ROWWIDTH = CE_PaperDollFrame:GetWidth()/2 - 10
local ROWHEIGHT = CE_PaperDollFrame:GetHeight()/NUMROWS

for i=1, 2*NUMROWS do
	local row = CreateFrame('Button', "CE_PaperDollRow"..i, CE_PaperDollFrame) -- base frame
	row:SetHeight(ROWHEIGHT)
	row:SetWidth(ROWWIDTH*(i<=NUMROWS and 0.88 or 1.12))
	row:SetPoint("TOP", i%NUMROWS == 1 and CE_PaperDollFrame or rows[i-1], i%NUMROWS == 1 and "TOP" or "BOTTOM")
	local left = i <= NUMROWS
	row:SetPoint("LEFT", left and CE_PaperDollFrame or getglobal("CE_PaperDollRow"..i-NUMROWS), left and "LEFT" or "RIGHT", left and 0 or 12, 0)
	
	local Label = row:CreateFontString(nil, nil, "GameFontNormalSmall")
	
	Label:SetPoint('LEFT', row, "LEFT", GAP, 0)
	Label:SetJustifyH('LEFT')
	row.Label = Label
	_G[row:GetName().."Label"] = Label
	
	local StatText = row:CreateFontString(nil, nil, "GameFontNormalSmall")
	StatText:SetPoint('RIGHT', row, "RIGHT", -GAP, 0)
	StatText:SetJustifyH('RIGHT')
	row.StatText = StatText
	_G[row:GetName().."StatText"] = StatText
	
	row:SetScript("OnLeave", function()
		GameTooltip:Hide()
	end)
	rows[i] = row
end

local headers = {
	PLAYERSTAT_BASE_STATS,
	L["Resistances"],
	PLAYERSTAT_DEFENSES,
	PLAYERSTAT_MELEE_COMBAT,
	PLAYERSTAT_RANGED_COMBAT,
	PLAYERSTAT_SPELL_COMBAT,
}

local function SetHeader(statFrame, index)
	local label = _G[statFrame:GetName().."Label"];
	local text = _G[statFrame:GetName().."StatText"];
	text:Hide()
	label:ClearAllPoints()
	label:SetPoint("CENTER",statFrame)
	label:SetJustifyH("CENTER")
	label:SetFontObject("GameFontWhiteSmall")
	label:SetText(headers[index])
	label:Show()
end


-- Modified from Blizzard's PaperDollFrame_SetResistances
local function SetResistance(frame, index)
	local label = _G[frame:GetName().."Label"];
	local text = _G[frame:GetName().."StatText"];
	
	-- The DAMAGE_SCHOOL# variables are one ahead of the RESISTANCE#_NAME variables, and the whole resist API
	local resistanceName = _G["DAMAGE_SCHOOL"..index+1];
	local base, resistance, positive, negative = UnitResistance("player", index);
	local petBonus = ComputePetBonus( "PET_BONUS_RES", resistance );

	label:SetText(resistanceName..":")
	
	-- Color the value red if net penalty; green if net buff; white otherwise
	if( abs(negative) > positive ) then
		text:SetText(RED_FONT_COLOR_CODE..resistance..FONT_COLOR_CODE_CLOSE);
	elseif( abs(negative) == positive ) then
		text:SetText(resistance);
	else
		text:SetText(GREEN_FONT_COLOR_CODE..resistance..FONT_COLOR_CODE_CLOSE);
	end
		
	frame.tooltip = resistanceName.." "..resistance;
	
	-- Concat buff/debuff sum equation
	if ( positive ~= 0 or negative ~= 0 ) then
		frame.tooltip = frame.tooltip.. " ( "..HIGHLIGHT_FONT_COLOR_CODE..base;
		if( positive > 0 ) then
			frame.tooltip = frame.tooltip..GREEN_FONT_COLOR_CODE.." +"..positive;
		end
		if( negative < 0 ) then
			frame.tooltip = frame.tooltip.." "..RED_FONT_COLOR_CODE..negative;
		end
		frame.tooltip = frame.tooltip..FONT_COLOR_CODE_CLOSE.." )";
	end
		
	local unitLevel = UnitLevel("player");
	local MAX_LEVEL = 80
	unitLevel = max(unitLevel, 20);
	-- Calculate against a boss at max level; own level otherwise
	local magicConstant = (unitLevel == MAX_LEVEL and 510 or unitLevel*5)
	local decResist = resistance/(magicConstant+resistance)*100
	local minDecResist = floor(decResist/10)*10
	--local toNextMinimum = (510*(minDecResist+0.1) + 51)/(0.9-(minDecResist+0.1)) 
	
	frame.tooltip2 = format(L["Resisting %.2f%% %s damage."], decResist, resistanceName)
	frame.tooltip2 = frame.tooltip2 .."\n".. format(L["Resisting at least %.2f%% per hit."], minDecResist)
	if unitLevel == MAX_LEVEL then
		frame.tooltip2 = frame.tooltip2 .."\n|cff888888".. format(L["(Against level %d bosses)"],unitLevel+3) .."|r"
	end
		
	if( petBonus > 0 ) then
		frame.tooltip2 = frame.tooltip2 .. "\n" .. format(PET_BONUS_TOOLTIP_RESISTANCE, petBonus);
	end
end

-- This function echoes Blizzard's UpdatePaperdollStats; same idea, but with minor tweaks to apply to my setup
local lastUpdate, throttle = 0, 1
function Update()
-- Throttling was added as an attempt to fix a bug. Turned out the cause was just something stupid. Remove this if it causes issues.
	if not (GetTime() > lastUpdate + throttle) then
		lastUpdate = GetTime()
		return
	end
	
	for i, v in ipairs({1,8,14,21,28,34}) do
		local header = _G["CE_PaperDollRow".. v+0];
		local stat1 = _G["CE_PaperDollRow".. v+1];
		local stat2 = _G["CE_PaperDollRow".. v+2];
		local stat3 = _G["CE_PaperDollRow".. v+3];
		local stat4 = _G["CE_PaperDollRow".. v+4];
		local stat5 = _G["CE_PaperDollRow".. v+5];
		local stat6 = _G["CE_PaperDollRow".. v+6];

		-- reset any OnEnter scripts that may have been changed
		header:SetScript("OnEnter", nil)
		stat1:SetScript("OnEnter", PaperDollStatTooltip);
		stat2:SetScript("OnEnter", PaperDollStatTooltip);
		stat3:SetScript("OnEnter", PaperDollStatTooltip);
		stat4:SetScript("OnEnter", PaperDollStatTooltip);
		stat5:SetScript("OnEnter", PaperDollStatTooltip);
		stat6:SetScript("OnEnter", PaperDollStatTooltip);
		-- stat6:Show();

		SetHeader(header, i)
		if ( i == 1 ) then
			PaperDollFrame_SetStat(stat1, 1);
			PaperDollFrame_SetStat(stat2, 2);
			PaperDollFrame_SetStat(stat3, 3);
			PaperDollFrame_SetStat(stat4, 4);
			PaperDollFrame_SetStat(stat5, 5);
			PaperDollFrame_SetArmor(stat6);
		elseif ( i == 2 ) then
			-- Why is the order all fucked here? To match the order Blizzard gives resists in.
			SetResistance(stat1, 6)
			SetResistance(stat2, 2)
			SetResistance(stat3, 3)
			SetResistance(stat4, 4)
			SetResistance(stat5, 5)
		elseif ( i == 3 ) then
			PaperDollFrame_SetArmor(stat1);
			PaperDollFrame_SetDefense(stat2);
			PaperDollFrame_SetDodge(stat3);
			PaperDollFrame_SetParry(stat4);
			PaperDollFrame_SetBlock(stat5);
			PaperDollFrame_SetResilience(stat6);
		elseif ( i == 4 ) then
			PaperDollFrame_SetDamage(stat1);
			stat1:SetScript("OnEnter", CharacterDamageFrame_OnEnter);
			PaperDollFrame_SetAttackSpeed(stat2);
			PaperDollFrame_SetAttackPower(stat3);
			PaperDollFrame_SetRating(stat4, CR_HIT_MELEE);
			PaperDollFrame_SetMeleeCritChance(stat5);
			PaperDollFrame_SetExpertise(stat6);
		elseif ( i == 5 ) then
			PaperDollFrame_SetRangedDamage(stat1);
			stat1:SetScript("OnEnter", CharacterRangedDamageFrame_OnEnter);
			PaperDollFrame_SetRangedAttackSpeed(stat2);
			PaperDollFrame_SetRangedAttackPower(stat3);
			PaperDollFrame_SetRating(stat4, CR_HIT_RANGED);
			PaperDollFrame_SetRangedCritChance(stat5);
		elseif ( i == 6 ) then
			PaperDollFrame_SetSpellBonusDamage(stat1);
			stat1:SetScript("OnEnter", CharacterSpellBonusDamage_OnEnter);
			PaperDollFrame_SetSpellBonusHealing(stat2);
			PaperDollFrame_SetRating(stat3, CR_HIT_SPELL);
			PaperDollFrame_SetSpellCritChance(stat4);
			stat4:SetScript("OnEnter", CharacterSpellCritChance_OnEnter);
			PaperDollFrame_SetSpellHaste(stat5);
			PaperDollFrame_SetManaRegen(stat6);
		end
	end
end

hooksecurefunc(PaperDollFrame, "Show", function()
	CE_PaperDollFrame:Show()
	for _,v in pairs(rows) do
		v:Show()
	end
	if IsAddOnLoaded("GearScore") or IsAddOnLoaded("GearScoreLite") then
		GearScore2:ClearAllPoints()
		GearScore2:SetPoint("BOTTOM", CharacterFrameTab1, "TOP", 0, 6)
		PersonalGearScore:ClearAllPoints()
		PersonalGearScore:SetPoint("BOTTOMLEFT", GearScore2, "TOPLEFT", 0, 0)
	end
end)
hooksecurefunc("UpdatePaperdollStats",Update)