﻿local name, addon = ...;

--[[
Dear localizers: The following should be a list of every localizable string used by Chill Effect.
Please supply your translation in the pair of quotes after the equals sign. The line following each string contains a description of where
the string will be used, what it should convey to the user, or where it can be found on the UI.
Some aspects of Chill Effect, notably the Pets module, rely on EXACT tooltip matches; they are marked as such.
	-- PETS MODULE
	["very fast"] = "",
		-- EXACT. This phrase appears in the tooltip of most "epic" mounts (100% land mounts and 280% flying mounts)
	["extremely fast"] = "",
		-- EXACT. This phrase appears in the tooltip of most "legendary" flying mounts (310% flying mounts)
	["Wintergrasp"] = "",
		-- EXACT. This text is the zone label on your minimap in most of the Lake Wintergrasp zone.
	["Ahn'Qiraj"] = "",
		-- EXACT. This text is the zone label on your minimap in the Temple of Ahn'Qiraj (AQ40).
	["Tailoring"] = "",
		-- EXACT. This Profession name appears on your Skills tab.
	["Engineering"] = "",
		-- EXACT. This Profession name appears on your Skills tab.
	["Riding"] = "",
		-- EXACT. This text appears on your Skills tab.
	["Outland"] = "",
		-- EXACT. This word (and the next one) appears in the tooltips of most flying mounts.
	["Northrend"] = "",
		-- EXACT. This word (and the previous one) appears in the tooltips of most flying mounts.
	["This mount can't move very quickly on land, but she's a great swimmer."] = "",
		-- EXACT. This is the last sentence in the tooltip of the Sea Turtle mount (wowhead.com/item=46109)
	["Reagents:"] = "",
		-- EXACT. This text appears in tooltips before the list of items needed to use a spell, such as Winter's Little Helper (wowhead.com/spell=26541)
	["Name"] = "",
		-- Label for the first sortable column in the pets tab. Clicking here sorts mounts alphabetically by their name.
	["Notes"] = "",
		-- Label for the second sortable column in the pets tab. Clicking here sorts mounts alphabetically using any additional description the mounts have been given.
	["Speed"] = "",
		-- Label for the third sortable column in the pets tab. Clicking here sorts mounts by their movement speed.
	["Varies"] = "",
		-- Short descriptor for mounts that change speed and/or type, depending on your Riding skill or location.
	["Ground"] = "",
		-- Short descriptor for mounts that cannot fly.
	["Flying"] = "",
		-- Short descriptor for mounts that can fly.
	["Aquatic"] = "",
		-- Short descriptor for mounts that perform best in water, like the Sea Turtle (wowhead.com/item=46109)
	["Passengers: %d"] = "",
		-- Short description of the number of passengers a mount can carry. Include the string "%d" as a placeholder for the number.
		-- If reasonable, position the placeholder at the end of the string, and keep the string as short as possible.
	
	-- CHARACTER MODULE
	["Resisting %.2f%% %s damage."] = "",
		-- Brief description of the average percentage of elemental damage that the player will resist. Try to match the "flavor" or "feel" of Blizzard's tooltips.
		-- Include the string "%.2f%%" as a placeholder for the amount and the string "%s" as a placeholder for the name of the element.
	["Resisting at least %.2f%% per hit."] = "",
		-- Brief description of the minimum percentage of elemental damage that the player will resist. Try to match the "flavor" or "feel" of Blizzard's tooltips.
		-- Include the string "%.2f%%" as a placeholder for the amount.
	["(Against level %d bosses)"] = "",
		-- This disclaimer is added to the tooltip containing the above two strings if the player is at maximum level, similar to the notice Blizzard adds to the Dodge tooltip.
		-- Include the string "%d" as a placeholder for the level of a raid boss.
		
	-- SKILLS MODULE
	["Riding"] = "",
		-- IGNORE/PLACEHOLDER. This is identical to the exact match listed under Pets, but listed here as a reminder if the Pets module should change.
	["Weapon Skills"] = "",
		-- EXACT. This should match the header row on the default Skills pane.
	["Secondary Skills"] = "",
		-- EXACT. This should match the header row on the default Skills pane.
	["Professions"] = "",
		-- EXACT. This should match the header row on the default Skills pane.
	["Click here to unlearn this profession."] = "",
		-- A warning to the player that this button will allow him to unlearn the profession associated with the button.
	["You will be prompted for confirmation."] = "",
		-- Note that clicking the button will not immediately unlearn the profession, and that the player will be asked to confirm the choice.
	]]

local L = setmetatable(GetLocale() == "deDE" and {
	["very fast"] = "sehr schnelles",
	["extremely fast"] = "extrem schnelles",
	["Wintergrasp"] = "Tausendwintersee",
	["Ahn'Qiraj"] = "Ahn'Qiraj",
	["Tailoring"] = "Schneiderei",
	["Engineering"] = "Ingenieurskunst",
	["Outland"] = "Scherbenwelt",
	["Northrend"] = "Nordend",
	["This mount can't move very quickly on land, but she's a great swimmer."] = "Dieses Reittier ist an Land nicht besonders flink, aber ein großartiger Schwimmer.",
	["Riding"] = "Reiten",
	["Reagents:"] = "Reagenzien:",
	["Ground"] = "Land",
	["Flying"] = "Fliegende",
	["Aquatic"] = "Aquatischen",
	["Passengers: %d"] = "Passagieren: %d",
} or GetLocale() == "zhTW" and {
	["very fast"] = "速度非常快",
	["extremely fast"] = "速度極快",
	["Wintergrasp"] = "冬握湖",
	["Ahn'Qiraj"] = "安其拉",
	["Tailoring"] = "裁縫",
	["Engineering"] = "工程學",
	["Outland"] = "外域",
	["Northrend"] = "北裂境",
	["This mount can't move very quickly on land, but she's a great swimmer."] = "此坐騎在陸上的速度並不快，但她是絕佳的游泳高手。",
	["Riding"] = "騎術",
	["Reagents:"] = "材料：",
	["Land Mount"] = "地面坐騎",
	["Flying Mount"] = "飛行坐騎",
	["Aquatic Mount"] = "水中坐騎",
	["%d Passengers"] = "%d名乘客",
} or GetLocale() == "frFR" and {
	["very fast"] = "très rapide",
	["extremely fast"] = "extrêmement rapide",
	["Wintergrasp"] = "Joug-d'hiver",
	["Ahn'Qiraj"] = "Ahn'Qiraj",
	["Tailoring"] = "Couture",
	["Engineering"] = "Ingénierie",
	["Outland"] = "Outreterre",
	["Northrend"] = "Norfendre",
	["This mount can't move very quickly on land, but she's a great swimmer."] = "Cette monture n'avance pas très rapidement sur la terre, mais c'est une excellente nageuse.",
	["Riding"] = "Monte",
	["Reagents:"] = "Composants :",
	["Name"] = "Nom",
	["Notes"] = "Notes",
	["Speed"] = "Vitesse",
	["Varies"] = "Adaptable",
	["Ground"] = "Terrestre",
	["Flying"] = "Volante",
	["Aquatic"] = "Aquatique",
	["Passengers: %d"] = "Passagers: %d",
	["Resisting %.2f%% %s damage."] = "Résistance de %.2f%% aux dégâts %s.",
	["Resisting at least %.2f%% per hit."] = "Résiste au moins de %.2f%% par coup.",
	["(Against level %d bosses)"] = "(Contre les boss niveau %d)",
	["Professions"] = "Métiers",
	["Secondary Skills"] = "Compétences secondaires",
	["Weapon Skills"] = "Compétences d’armes",
	["Click here to unlearn this profession."] = "Oublier ce métier.",
	["You will be prompted for confirmation."] = "Une confirmation vous sera demandée.",
} or 
	{}, {__index=function(t,i) return i end})
addon.L = L
