local name, addon = ...;
local L = addon.L;
local GetExtraCompanionData = addon.GetExtraCompanionData
local GetCompanionInfo, GetNumCompanions = GetCompanionInfo, GetNumCompanions
local MOUNTTYPE_LAND = addon.MOUNTTYPE_LAND
local MOUNTTYPE_FLYING = addon.MOUNTTYPE_FLYING
local MOUNTTYPE_AQUATIC = addon.MOUNTTYPE_AQUATIC

-- Utilty function. 
local function addLine(base, new)
	if base and base ~= "" then
		return base .. "\n" .. new
	else
		return new
	end
end

local NUMROWS = 13
local ROWHEIGHT = 24
local ROWMARGIN = 1.5
local ROWPADDING = 2
local SCROLLSTEP = 5
local ROTATETIME = 6 -- Number of seconds it takes for the 3D preview to rotate once on its axis.

-- For some reason, the 12 CompanionButtons on the default UI are protected, which implicitly protects their parent.
for i=1,12 do
	SecureHandlerWrapScript(_G["CompanionButton"..i], "OnShow", _G["CompanionButton"..i], [[ self:GetParent():Hide() ]])
end

local CE_Pets = CreateFrame("Frame", "CE_Pets", PetPaperDollFrame)
CE_Pets:SetWidth(320) -- Consider a better way to do this
CE_Pets:SetHeight(NUMROWS*(ROWHEIGHT+ROWMARGIN)-ROWMARGIN)
CE_Pets:SetPoint("TOPLEFT", 21, -100)
CE_Pets:Hide()

local scrollbar = LibStub("tekKonfig-Scroll").new(CE_Pets, 0, SCROLLSTEP)
scrollbar:SetPoint("TOPRIGHT", 0, -14)
CE_Pets.scrollbar = scrollbar


CE_Pets:EnableMouseWheel(true)
CE_Pets:SetScript("OnMouseWheel", function(self, value)
	scrollbar:SetValue(scrollbar:GetValue() - value * SCROLLSTEP)
end)

local CE_Pets3DModel = CreateFrame("DressUpModel")
CE_Pets3DModel:SetFrameStrata("TOOLTIP")
CE_Pets3DModel:SetBackdrop({
	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
	tile = nil, tileSize = 0, edgeSize = 10,
	insets = { left = 3, right = 3, top = 3, bottom = 3 }
})
CE_Pets3DModel:SetBackdropColor(0.3, 0.3, 0.3 ,1)
CE_Pets3DModel:Hide()
CE_Pets3DModel:SetScript("OnShow", function(self) self:SetFacing(0) end)
CE_Pets3DModel:SetScript("OnUpdate", function(self, e) self:SetFacing(self:GetFacing() + e*math.pi/ROTATETIME) end)

local function RowOnClick(self, button)
	if IsModifiedClick("CHATLINK") then
		local spellId = select(3, GetCompanionInfo(CE_Pets.COMPANIONTYPE, self:GetID()))
		ChatEdit_InsertLink(GetSpellLink(spellId));
	elseif not IsFlying() then -- Protect against aerial dismounts
		CancelShapeshiftForm() -- This fails silently if the player is not "shapeshifted"; mostly druids, ghost wolf
		CallCompanion(CE_Pets.COMPANIONTYPE,self:GetID())
	end
end

local RowOnEnter = function(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	local spell = select(3, GetCompanionInfo(CE_Pets.COMPANIONTYPE, self:GetID()))
	GameTooltip:SetHyperlink(GetSpellLink(spell))
	GameTooltip:Show()
	
	CE_Pets3DModel:SetPoint("TOPLEFT",GameTooltip,"BOTTOMLEFT")
	CE_Pets3DModel:SetPoint("TOPRIGHT",GameTooltip,"BOTTOMRIGHT")
	local size = GameTooltip:GetWidth()
	CE_Pets3DModel:SetPoint("BOTTOMLEFT",GameTooltip,"BOTTOMLEFT", 0, -size/2)
	CE_Pets3DModel:SetPoint("BOTTOMRIGHT",GameTooltip,"BOTTOMRIGHT", 0, -size/2)
	
	CE_Pets3DModel:SetCreature(GetCompanionInfo(CE_Pets.COMPANIONTYPE,self:GetID()))
	CE_Pets3DModel:Show()
end

local RowOnLeave = function()
	GameTooltip:Hide()
	CE_Pets3DModel:Hide()
	ResetCursor()
end



do
	--local width = CE_Pets:GetWidth() - 16 -- scrollbar is 16px, plus a bit for padding

	local header1 = CreateFrame("Button", "CE_PetsHeader1", CE_Pets, "WhoFrameColumnHeaderTemplate")
	header1:SetPoint("BOTTOMLEFT", CE_Pets, "TOPLEFT", 0, 2)
	--WhoFrameColumn_SetWidth(header1, width*0.65)
	header1.TYPE = "NAME"
	header1:SetText(L["Name"])
	
	local header2 = CreateFrame("Button", "CE_PetsHeader2", CE_Pets, "WhoFrameColumnHeaderTemplate")
	header2:SetPoint("TOPLEFT", header1, "TOPRIGHT", 0, 0)
	--WhoFrameColumn_SetWidth(header2, width*0.185) -- cut a couple of pixels for padding
	header2.TYPE = "NOTES"
	header2:SetText(L["Notes"])
	
	local header3 = CreateFrame("Button", "CE_PetsHeader3", CE_Pets, "WhoFrameColumnHeaderTemplate")
	header3:SetPoint("TOPLEFT", header2, "TOPRIGHT", 0, 0)
	--WhoFrameColumn_SetWidth(header3, width*0.165) -- cut a couple of pixels for padding
	header3.TYPE = "SPEED"
	header3:SetText(L["Speed"])
end


-- Build the rows
local rows = {}
for i=1,NUMROWS do
	local width = CE_Pets:GetWidth() - 16
	local row = CreateFrame("Button", "CE_PetsRow"..i, CE_Pets)
	row:SetHeight(ROWHEIGHT)
	if i == 1 then
		row:SetPoint("TOP", CE_Pets, "TOP", 0, 1)
	else
		row:SetPoint("TOP", rows[i-1], "BOTTOM", 0, -ROWMARGIN)
	end
	row:SetPoint("LEFT")
	row:SetWidth(width)

	-- This is the same higlight that Blizzard uses in the Currencies pane
	row:SetHighlightTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	row:GetHighlightTexture():SetTexCoord(0, 1, 0.609375, 0.796875)
	
	local stripe = row:CreateTexture(nil, "BACKGROUND")
	stripe:SetAllPoints()
	stripe:SetTexture(1, 1, 1, 0.06)
	row.Stripe = stripe
	
	-- backdrop:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	-- backdrop:SetTexCoord(0, 1, 0.609375, 0.796875)
	
	-- local backdrop = row:CreateTexture(nil, "BACKGROUND")
	-- backdrop:SetPoint("TOPRIGHT")
	-- backdrop:SetPoint("BOTTOMLEFT", 2, 0)
	-- backdrop:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background")
	-- backdrop:SetGradientAlpha("HORIZONTAL", 1,0,0,0.75, 1,0,0,0)
	-- row.Backdrop = backdrop
	
		local iconSize = ROWHEIGHT - ROWPADDING
		local icon = CreateFrame("Frame", nil, row)
		icon:SetPoint("LEFT", row, "LEFT", ROWPADDING*2, 0)
		icon:SetHeight(iconSize)
		icon:SetWidth(iconSize)
		row.icon = icon; --NOTE: I=Texture, i=Frame
		row.Icon = icon:CreateTexture(nil, "BORDER")
		row.Icon:SetAllPoints()
		
		local index = icon:CreateFontString(nil, "OVERLAY", "NumberFontNormalSmall")
		index:SetJustifyH("RIGHT")
		index:SetPoint("BOTTOMRIGHT", icon, "BOTTOMRIGHT", 2, 0)
		row.Index = index
		
		local Name = row:CreateFontString(nil, nil, "GameFontHighlightSmallLeft")
		Name:SetPoint('LEFT', icon, "RIGHT", ROWPADDING, 0)
		Name:SetWordWrap(true)
		row.Name = Name

		local Notes = row:CreateFontString(nil, nil, "GameFontHighlightSmallLeft")
		Notes:SetJustifyH("LEFT")
		Notes:SetPoint("LEFT", Name, "RIGHT", 8, 0)
		row.Notes = Notes
	
		local Speed = row:CreateFontString(nil, nil, "GameFontHighlightSmallLeft")
		Speed:SetJustifyH("LEFT")
		Speed:SetPoint("LEFT", Notes, "RIGHT", 0, 0)
		row.Speed = Speed

	row:SetScript('OnEnter', RowOnEnter)
	row:SetScript('OnLeave', RowOnLeave)
	row:SetScript('OnClick', RowOnClick)
	row:SetScript('OnDragStart', function(self, button)
		PickupCompanion(CE_Pets.COMPANIONTYPE, self:GetID())
	end)
	row:RegisterForDrag("LeftButton","RightButton")
	
	row.update = function(self, data, rownum)
		row.Icon:SetTexture(data.ICON)
		row.Index:SetText(data.ID)
		row.Name:SetText(data.NAME or L["<Loading companion data>"])
		if row.Notes:IsShown() then
			row.Notes:SetText(data.NOTES)
		end
		if row.Speed:IsShown() then
			row.Speed:SetFormattedText("%d%%", data.SPEED or 0)
		end
		row:SetID(data.ID)
		if data.ACTIVE then
			row:LockHighlight()
		else
			row:UnlockHighlight()
		end
		if data.ISUSABLE() then
			row.Icon:SetVertexColor(1.0, 1.0, 1.0)
			-- row.Index:SetFontObject("NumberFontNormalSmall")
			-- row.Name:SetFontObject("GameFontHighlightSmallLeft")
			-- row.Notes:SetFontObject("GameFontHighlightSmallLeft")
			-- row.Speed:SetFontObject("GameFontHighlightSmallLeft")
			row.Index:SetAlpha(1.0)
			row.Name:SetAlpha(1.0)
			row.Notes:SetAlpha(1.0)
			row.Speed:SetAlpha(1.0)
		else
			row.Icon:SetVertexColor(0.2, 0.2, 0.2)
			-- row.Index:SetFontObject("NumberFontNormalSmallGray")
			-- row.Name:SetFontObject("GameFontDisableSmallLeft")
			-- row.Notes:SetFontObject("GameFontDisableSmallLeft")
			-- row.Speed:SetFontObject("GameFontDisableSmallLeft")
			row.Index:SetAlpha(0.5)
			row.Name:SetAlpha(0.4)
			row.Notes:SetAlpha(0.4)
			row.Speed:SetAlpha(0.4)
		end
		if mod(rownum,2) == 1 then
			row.Stripe:Show()
		else
			row.Stripe:Hide()
		end
	end
	
	rows[i] = row
end

local MountData = {}
local creatureName, spellID, icon, active, speed, mountType, numPassengers, isUsable, varies, notes, updated
local function UpdateMountData()
	for i=1, GetNumCompanions("MOUNT") do
		notes = ""
		_, creatureName, spellID, icon, active = GetCompanionInfo("MOUNT", i)
		speed, mountType, numPassengers, isUsable, varies = unpack(GetExtraCompanionData("MOUNT", spellID),1,5)
		
		if numPassengers > 0 then
			notes = addLine(notes, format(L["Passengers: %d"], numPassengers))
		end
		
		if varies then
			notes = addLine(notes, L["Varies"])
		end
		
		if not strfind(notes, "\n") then
			if mountType == MOUNTTYPE_LAND then
				notes = addLine(notes, L["Ground"])
			elseif mountType == MOUNTTYPE_FLYING then
				notes = addLine(notes, L["Flying"])
			elseif mountType == MOUNTTYPE_AQUATIC then
				notes = addLine(notes, L["Aquatic"])
			end
		end
		
		updated = false
		for _,v in pairs(MountData) do
			if v.NAME == creatureName then
				v.ACTIVE = active
				v.SPEED = speed
				v.NOTES = notes
				v.ISUSABLE = isUsable
				v.ID = i
				updated = true
			end
		end
		if not updated then
			tinsert(MountData, {NAME = creatureName, NOTES = notes, SPEED = speed, ICON = icon, ACTIVE = active, ISUSABLE = isUsable, ID = i})
		end
	end
end

local CritterData = {}
local function UpdateCritterData()
	for i=1, GetNumCompanions("CRITTER") do
		_, creatureName, spellID, icon, active = GetCompanionInfo("CRITTER", i)
		isUsable = GetExtraCompanionData("CRITTER", spellID)
		
		updated = false
		for _,v in pairs(CritterData) do
			if v.NAME == creatureName then
				v.ACTIVE = active
				v.ISUSABLE = isUsable
				v.ID = i
				updated = true
			end
		end
		if not updated then
			tinsert(CritterData, {NAME = creatureName, ICON = icon, ACTIVE = active, ISUSABLE = isUsable, ID = i})
		end
	end
end

local offset = 0
function CE_Pets:RefreshDisplay()
	assert(#self.DATA > 0, "Companion list not built")
	
	local width
	if (#self.DATA > NUMROWS) then
		self.scrollbar:Show()
		width = self:GetWidth() - self.scrollbar:GetWidth() - 4
	else
		self.scrollbar:Hide()
		width = self:GetWidth()
	end
	
	if self.COMPANIONTYPE == "CRITTER" then
		WhoFrameColumn_SetWidth(CE_PetsHeader1, width)
		CE_PetsHeader2:Hide()
		CE_PetsHeader3:Hide()
		for _,row in pairs(rows) do
			row:SetWidth(width)
			row.Name:SetWidth(CE_PetsHeader1:GetWidth() - row.icon:GetWidth() - ROWPADDING*3)
			row.Notes:Hide()
			row.Speed:Hide()
		end
	else
		WhoFrameColumn_SetWidth(CE_PetsHeader1, width*0.58)
		WhoFrameColumn_SetWidth(CE_PetsHeader2, width*0.26)
		WhoFrameColumn_SetWidth(CE_PetsHeader3, width*0.16)
		CE_PetsHeader2:Show()
		CE_PetsHeader3:Show()
		for _,row in pairs(rows) do
			row:SetWidth(width)
			row.Name:SetWidth(CE_PetsHeader1:GetWidth() - row.icon:GetWidth() - ROWPADDING*3)
			row.Notes:SetWidth(CE_PetsHeader2:GetWidth())
			row.Notes:Show()
			row.Speed:SetWidth(CE_PetsHeader3:GetWidth())
			row.Speed:Show()
		end
	end
	
	for i,row in pairs(rows) do
		local j = i + offset
		if j > #self.DATA then
			row:Hide()
		else
			row:update(self.DATA[j], j)
			row:Show()
		end
	end
end

local f = scrollbar:GetScript("OnValueChanged")
scrollbar:SetScript("OnValueChanged", function(self, value, ...)
	offset = math.floor(value)
	self:GetParent():RefreshDisplay()
	if strfind((GetMouseFocus() and GetMouseFocus():GetName() or ""), "CE_PetsRow%d") then
		GetMouseFocus():GetScript("OnEnter")(GetMouseFocus())
	end
	return f(self, value, ...)
end)

function CE_Pets:SetCompanionType(type)
	assert(type=="CRITTER" or type=="MOUNT", "Unknown companion type")
	if type == "MOUNT" then
		self.DATA = MountData
	else
		self.DATA = CritterData
	end
	self.COMPANIONTYPE = type
end

local ascending = function(a, b)
	return a[CE_Pets.SORTTYPE] < b[CE_Pets.SORTTYPE] or a[CE_Pets.SORTTYPE]==b[CE_Pets.SORTTYPE] and a["NAME"]<b["NAME"]
end
local descending = function(a, b)
	return a[CE_Pets.SORTTYPE] > b[CE_Pets.SORTTYPE] or a[CE_Pets.SORTTYPE]==b[CE_Pets.SORTTYPE] and a["NAME"]<b["NAME"]
end
	
function CE_Pets:SortCompanionData(type, reset, refresh)
	assert(type=="NAME" or type=="NOTES" or type=="SPEED", format("Unknown sorting type: %s",type))
	local sortFunction
	if reset then
		self.SORTTYPE = "NAME"
		self.SORTFUNC = ascending
		table.sort(self.DATA, self.SORTFUNC)
	elseif refresh then
		table.sort(self.DATA, self.SORTFUNC)
	else
		if type == self.SORTTYPE then -- If it's the same tab as the current one, reverse the order
			if self.SORTFUNC == ascending then
				sortFunction = descending
			else
				sortFunction = ascending
			end
		else -- New sorting order. If we're trying to go to Speed, we want descending (Fast to slow)
			if type == "SPEED" then
				sortFunction = descending
			else
				sortFunction = ascending
			end
		end
		self.SORTTYPE = type
		self.SORTFUNC = sortFunction
		table.sort(self.DATA, sortFunction)
	end
end



local oldtex = {}
CE_Pets:SetScript("OnShow", function(self)
	-- On the first show, query, set type, and sort if needed.
	if #self.DATA == 0 then
		UpdateMountData()
		UpdateCritterData()
		if GetNumCompanions("CRITTER") == 0 then
			CE_Pets:SetCompanionType("MOUNT")
		else
			CE_Pets:SetCompanionType("CRITTER")
		end
		CE_Pets:SortCompanionData("NAME")
	end
	CE_Pets:RefreshDisplay()
	
	scrollbar:SetMinMaxValues(0, math.max(0, #self.DATA - NUMROWS))
	scrollbar:SetValue(0)
	
	-- The texturing for the pets tab needs to be changed in order to make room for the new sorting buttons.
	-- The old ones are stored in the oldtex table and restored OnHide.
	for k,v in pairs({PetPaperDollFrame:GetRegions()}) do
		if v:GetObjectType()=="Texture" then
			if strfind(v:GetTexture(), "TopLeft") then
				oldtex[v] = v:GetTexture(); v:SetTexture("Interface\\ClassTrainerFrame\\UI-ClassTrainer-TopLeft")
			elseif strfind(v:GetTexture(), "TopRight") then
				oldtex[v] = v:GetTexture(); v:SetTexture("Interface\\ClassTrainerFrame\\UI-ClassTrainer-TopRight")
			elseif strfind(v:GetTexture(), "BotLeft") then
				oldtex[v] = v:GetTexture(); v:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-BottomLeft")
			elseif strfind(v:GetTexture(), "BotRight") then
				oldtex[v] = v:GetTexture(); v:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-BottomRight")
			end
		end
	end
end)
CE_Pets:SetScript("OnHide", function(self)
	for v, t in pairs(oldtex) do
		v:SetTexture(t)
	end
end)

CE_Pets:RegisterEvent("COMPANION_UPDATE")
CE_Pets:RegisterEvent("COMPANION_LEARNED")
CE_Pets:RegisterEvent("SKILL_LINES_CHANGED")
local skills_available = false
CE_Pets:SetScript("OnEvent", function(self, event)
	if event == "SKILL_LINES_CHANGED" then skills_available = true end
	if not skills_available then return end
	UpdateMountData()
	UpdateCritterData()
	if self:IsShown() then
		self:SetCompanionType(self.COMPANIONTYPE)
		if not self.SORTTYPE then
			self:SortCompanionData("NAME")
		else
			self:SortCompanionData(self.SORTTYPE, nil, true)
		end
		self:RefreshDisplay()
	end
end)

hooksecurefunc("PetPaperDollFrame_UpdateTabs", function()
	if PanelTemplates_GetSelectedTab(PetPaperDollFrame) == 2 then
		CE_Pets:SetCompanionType("CRITTER")
		CE_Pets:Show()
	elseif PanelTemplates_GetSelectedTab(PetPaperDollFrame) == 3 then
		CE_Pets:SetCompanionType("MOUNT")
		CE_Pets:Show()
	end
end)

if PetPaperDollFrame:IsVisible() then Show() end -- This line runs only if the frame is open when the addon loads. o.0

-- "Pet" button
PetPaperDollFrameTab1:HookScript("OnClick", function(self)
	CE_Pets:Hide();
end)

-- "Companions" button
PetPaperDollFrameTab2:HookScript("OnClick", function(self)
	CE_Pets:SetCompanionType("CRITTER")
	CE_Pets:SortCompanionData("NAME", true)
	if CE_Pets:IsShown() then
		CE_Pets:RefreshDisplay()
	else
		CE_Pets:Show()
	end
	scrollbar:SetMinMaxValues(0, math.max(0, GetNumCompanions(CE_Pets.COMPANIONTYPE) - NUMROWS))
	scrollbar:SetValue(0)
end)

-- "Mounts" button
PetPaperDollFrameTab3:HookScript("OnClick", function(self)
	CE_Pets:SetCompanionType("MOUNT")
	CE_Pets:SortCompanionData("NAME", true)
	if CE_Pets:IsShown() then
		CE_Pets:RefreshDisplay()
	else
		CE_Pets:Show()
	end

	scrollbar:SetMinMaxValues(0, math.max(0, GetNumCompanions(CE_Pets.COMPANIONTYPE) - NUMROWS))
	scrollbar:SetValue(0)
end)


local SortTabOnClick = function(self)
	CE_Pets:SortCompanionData(self.TYPE)
	CE_Pets:RefreshDisplay()
end

CE_PetsHeader1:SetScript("OnClick", SortTabOnClick)
CE_PetsHeader2:SetScript("OnClick", SortTabOnClick)
CE_PetsHeader3:SetScript("OnClick", SortTabOnClick)